from flask import render_template, request, redirect, url_for, flash
from flask_login import login_user, logout_user, login_required, current_user
from . import auth
from .. import db
from ..models import User

from .forms import LoginForm, RegistrationForm


@auth.route("/")
def home():
    return render_template("splash.html", style_sheet="login.css")


@auth.before_app_request
def before_request():
    if current_user.is_authenticated:
        if not current_user.confirmed \
                and request.endpoint \
                and request.blueprint != 'auth' \
                and request.endpoint != 'static':
            return redirect(url_for('auth.unconfirmed'))


@auth.route('/unconfirmed')
def unconfirmed():
    if current_user.is_anonymous or current_user.confirmed:
        return redirect(url_for('home'))
    # todo make this page
    flash("Please confirm your email address")
    return redirect(url_for("auth.login"))


@auth.route("/login", methods=['GET', 'POST'])
def login():
    login_form = LoginForm()
    registration_form = RegistrationForm()
    if login_form.validate_on_submit():
        user = User.query.filter_by(username=login_form.username.data).first()
        if user is not None and user.verify_password(login_form.password.data):
            login_user(user)
            next = request.args.get('next')
            if next is None or not next.startswith('/'):
                next = url_for('my_files.myfiles', parent_dir="/")
            return redirect(next)
    if login_form.is_submitted():
        error_list = list(login_form.errors.keys())
        if len(error_list) > 0:
            if "username" in error_list:
                flash(login_form.errors["username"][0])

    return render_template("login.html", mode="login", style_sheet="login.css",
                           login=login_form, register=registration_form)


@auth.route("/register", methods=['GET', 'POST'])
def register():
    login_form = LoginForm()
    registration_form = RegistrationForm()

    if registration_form.validate_on_submit():
        user = User(email=registration_form.email.data.lower(),
                    username=registration_form.username.data,
                    password_hash=registration_form.password.data)
        db.session.add(user)
        db.session.commit()

        token = user.generate_confirmation_token()
        user.send_email_confirmation(email=user.email, token=token)
        flash("Confirmation email sent")

    if registration_form.is_submitted():
        error_list = list(registration_form.errors.keys())
        if len(error_list) > 0:
            if "username" in error_list:
                flash(registration_form.errors["username"][0])
            elif "email" in error_list:
                flash(registration_form.errors["email"][0])
            elif "password" in error_list:
                flash("Incorrect Password")
            elif "password2" in error_list:
                flash("Password fields must match")

    return render_template("login.html", mode="register", style_sheet="login.css",
                           login=login_form, register=registration_form)


@auth.route('/logout')
@login_required
def logout():
    logout_user()
    flash('You have been logged out.')
    return redirect(url_for("auth.login"))


@auth.route('/confirm_email/<user_id>/<token>')
@login_required
def confirm_email(user_id, token):
    user = User.query.filter_by(id=user_id).first()
    if user.confirmed:
        return redirect(url_for('my_files.myfiles'))
    if user.confirm(token):
        db.session.commit()
        flash('You have confirmed your account. Thanks!')
    else:
        flash('The confirmation link is invalid or has expired.')
    return redirect(url_for('my_files.myfiles'))


@login_required
@auth.route('/user/<user_id>')
def user_profile(user_id):
    user = User.query.filter_by(id=user_id).first()
    # todo make this page
    return redirect(url_for("auth.home"))